#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const yargs_1 = __importDefault(require("yargs"));
const os_1 = __importDefault(require("os"));
const configuring_1 = require("configuring");
const micro_1 = __importDefault(require("micro"));
const default_1 = __importDefault(require("../config/default"));
const logger_1 = __importDefault(require("../logger"));
const index_1 = __importDefault(require("../index"));
['SIGINT', 'SIGTERM'].forEach((signal) => {
    process.on(signal, () => process.exit(0));
});
const options = 
// .help('Caravaggio')
yargs_1.default
    .option('port', {
    alias: 'p',
    default: process.env.PORT ? parseInt(process.env.PORT) : 8565,
    describe: 'the server will listen to this port',
    type: 'number',
})
    .option('cache', {
    default: 'memory',
    describe: 'Set the cache system',
    type: 'string',
    choices: ['memory', 'file', 'none'],
    group: 'cache',
})
    .option('cache-filepath', {
    describe: '[when --cache=file] The file path for the file cache',
    type: 'string',
    default: os_1.default.tmpdir(),
    group: 'cache',
})
    .option('cache-limit', {
    default: 100,
    describe: '[when --cache=memory] The maximum amount of memory to use in MB',
    type: 'number',
    group: 'cache',
})
    .option('inputcache', {
    default: 'none',
    describe: 'Set the input cache system. This cache save the original images to avoid re-download',
    type: 'string',
    choices: ['memory', 'file', 'none'],
    group: 'input cache',
})
    .option('inputcache-filepath', {
    describe: '[when --cache=file] The file path for the file cache',
    type: 'string',
    default: os_1.default.tmpdir(),
    group: 'input cache',
})
    .option('inputcache-limit', {
    default: 100,
    describe: '[when --cache=memory] The maximum amount of memory to use in MB',
    type: 'number',
    group: 'input cache',
})
    .option('whitelist', {
    describe: 'Restrict the images to a list of domains',
    type: 'array',
    group: 'security',
})
    .option('verbose', {
    alias: 'v',
    describe: 'Increase the verbosity of the application',
    type: 'boolean',
    // conflicts: 'quiet',
    group: 'log',
})
    .option('quiet', {
    alias: 'q',
    describe: 'Do not output anything',
    type: 'boolean',
    // conflicts: 'verbose',
    group: 'log',
})
    .option('json', {
    describe: 'Output logs in json format.',
    type: 'boolean',
    default: false,
    group: 'log',
})
    // .conflicts('verbose', 'quiet')
    .options('progressive', {
    describe: 'the output images are progressive by default (when applicable)',
    type: 'boolean',
    default: true,
    group: 'image manipulation',
})
    .options('errors', {
    describe: 'set the error output format',
    type: 'string',
    choices: ['plain', 'html', 'json'],
    default: 'json',
    group: 'misc',
})
    .options('compress', {
    describe: 'compress the response through gzip/deflate/brotli if the browser asks for it',
    type: 'boolean',
    default: false,
    group: 'misc',
}).argv;
const defaultOperations = [];
defaultOperations.push({
    operation: 'progressive',
    value: `${options.progressive}`,
});
let cacheOptions;
switch (options.cache) {
    case 'file':
        cacheOptions = {
            basePath: options['cache-filepath'],
        };
        break;
    case 'memory':
        cacheOptions = {
            limit: options['cache-limit'],
        };
        break;
    default:
        cacheOptions = {};
        break;
}
let inputCacheOptions;
switch (options.inputcache) {
    case 'file':
        inputCacheOptions = {
            basePath: options['inputcache-filepath'],
        };
        break;
    case 'memory':
        inputCacheOptions = {
            limit: options['inputcache-limit'],
        };
        break;
    default:
        inputCacheOptions = {};
        break;
}
const cliConfig = {
    caches: {
        output: {
            type: options.cache,
            options: cacheOptions,
        },
        input: {
            type: options.inputcache,
            options: inputCacheOptions,
        },
    },
    logger: {
        options: {
            prettyPrint: !options.json,
            level: options.verbose ? 'debug' : options.quiet ? 'silent' : 'info',
        },
    },
    whitelist: options.whitelist || [],
    errors: options.errors,
    compress: options.compress,
    defaultOperations,
};
const configManager = (0, configuring_1.createConfigManager)({
    configurations: {
        default: default_1.default,
        cli: cliConfig,
    },
});
const config = configManager.getConfig('cli');
const { port } = options;
const logger = (0, logger_1.default)(config);
const server = (0, micro_1.default)((0, index_1.default)(config));
logger.debug(Object.assign(Object.assign({}, config), { logger: Object.assign(Object.assign({}, config.logger), { destination: 'stdout' }) }));
server.listen(port);
logger.info(`Caravaggio started on port ${port}. Preview at http://localhost:${port}`);
