"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const cohercer_1 = __importDefault(require("../utils/cohercer"));
const colors_1 = require("../utils/colors");
const sharp_1 = __importDefault(require("sharp"));
const misc_1 = require("../utils/misc");
// Implementation from gatsby-plugin-sharp
// @see https://github.com/gatsbyjs/gatsby/blob/bb0bb9c/packages/gatsby-plugin-sharp/src/duotone.js
const errorMessage = 'Invalid color parameter';
const docUri = 'duotone.html';
const createGradient = (primaryColorRGB, secondaryColorRGB) => {
    const gradient = [];
    for (let i = 0; i < 256; i++) {
        const ratio = i / 255;
        gradient.push([
            Math.round(primaryColorRGB.r * ratio + secondaryColorRGB.r * (1 - ratio)),
            Math.round(primaryColorRGB.g * ratio + secondaryColorRGB.g * (1 - ratio)),
            Math.round(primaryColorRGB.b * ratio + secondaryColorRGB.b * (1 - ratio)),
        ]);
    }
    return gradient;
};
/**
 * Create a semirtansparent, mono-channel version of the duotone imag and apply as alpha channel
 * to the original image
 * @param {Sharp} duotoneImage Duotone image
 * @param {Sharp} originalImage Original image that produced the duotone image
 * @param {Number} opacity A valu between > 0 and < 1
 * @param {String} format The resulting format
 */
const overlayAlphaImage = (duotoneImage, originalImage, opacity, format) => __awaiter(void 0, void 0, void 0, function* () {
    const metadata = yield duotoneImage.flatten().metadata();
    const width = metadata.width;
    const height = metadata.height;
    const percentGrey = Math.round(opacity * 255);
    const percentTransparency = Buffer.alloc(width * height, percentGrey);
    const duotoneWithTransparency = yield duotoneImage
        .joinChannel(percentTransparency, {
        raw: { width, height, channels: 1 },
    })
        .raw()
        .toBuffer();
    return originalImage
        .composite([
        {
            input: duotoneWithTransparency,
            raw: { width, height, channels: 4 },
        },
    ])
        .raw()
        .toBuffer({ resolveWithObject: true })
        .then(({ data, info: raw }) => (0, sharp_1.default)(data, { raw: raw }).toFormat(format));
});
const duotone = ({ h: highlightColor, s: shadowColor, o: opacity, }) => {
    const gradient = createGradient((0, colors_1.getColorFromParameter)(highlightColor, errorMessage, docUri), (0, colors_1.getColorFromParameter)(shadowColor, errorMessage, docUri));
    let opacityValue = null;
    if (opacity) {
        opacityValue = (0, cohercer_1.default)(opacity, 'Opacity is not valid', docUri)
            .toFloat()
            .value();
    }
    const params = [highlightColor, shadowColor];
    if (opacityValue) {
        params.push(opacityValue);
    }
    return [
        {
            name: 'duotone',
            params,
            op: ({ image, otherOps }) => __awaiter(void 0, void 0, void 0, function* () {
                const format = yield (0, misc_1.getOutputType)(image, otherOps);
                const { data, info } = yield image
                    .clone()
                    .raw()
                    .toBuffer({ resolveWithObject: true });
                for (let i = 0; i < data.length; i += info.channels) {
                    const r = data[i];
                    const g = data[i + 1];
                    const b = data[i + 2];
                    // @see https://en.wikipedia.org/wiki/Relative_luminance
                    const avg = Math.round(0.2126 * r + 0.7152 * g + 0.0722 * b);
                    [data[i], data[i + 1], data[i + 2]] = gradient[avg];
                }
                const newImage = (0, sharp_1.default)(data, { raw: info });
                if (opacityValue && opacityValue > 0 && opacityValue < 1) {
                    return overlayAlphaImage(newImage.clone(), image.clone(), opacityValue, format);
                }
                return format ? newImage.toFormat(format) : newImage;
            }),
        },
    ];
};
exports.default = duotone;
